@@ -7,12 +7,13 @@ import logging
 from pathlib import Path
 from typing import Dict
 
-from freqtrade.constants import DEFAULT_HYPEROPT_LOSS, USERPATH_HYPEROPTS
+from freqtrade.constants import HYPEROPT_LOSS_BUILTIN, USERPATH_HYPEROPTS
 from freqtrade.exceptions import OperationalException
 from freqtrade.optimize.hyperopt_interface import IHyperOpt
 from freqtrade.optimize.hyperopt_loss_interface import IHyperOptLoss
 from freqtrade.resolvers import IResolver
 
+
 logger = logging.getLogger(__name__)
 
 
@@ -69,10 +70,13 @@ class HyperOptLossResolver(IResolver):
         :param config: configuration dictionary
         """
 
-        # Verify the hyperopt_loss is in the configuration, otherwise fallback to the
-        # default hyperopt loss
-        hyperoptloss_name = config.get('hyperopt_loss') or DEFAULT_HYPEROPT_LOSS
-
+        hyperoptloss_name = config.get('hyperopt_loss')
+        if not hyperoptloss_name:
+            raise OperationalException(
+                "No Hyperopt loss set. Please use `--hyperopt-loss` to "
+                "specify the Hyperopt-Loss class to use.\n"
+                f"Built-in Hyperopt-loss-functions are: {', '.join(HYPEROPT_LOSS_BUILTIN)}"
+            )
         hyperoptloss = HyperOptLossResolver.load_object(hyperoptloss_name,
                                                         config, kwargs={},
                                                         extra_dir=config.get('hyperopt_path'))
@@ -81,8 +85,4 @@ class HyperOptLossResolver(IResolver):
         hyperoptloss.__class__.ticker_interval = str(config['timeframe'])
         hyperoptloss.__class__.timeframe = str(config['timeframe'])
 
-        if not hasattr(hyperoptloss, 'hyperopt_loss_function'):
-            raise OperationalException(
-                f"Found HyperoptLoss class {hyperoptloss_name} does not "
-                "implement `hyperopt_loss_function`.")
         return hyperoptloss
