@@ -6,7 +6,6 @@ This module load custom strategies
 import logging
 import tempfile
 from base64 import urlsafe_b64decode
-from collections import OrderedDict
 from inspect import getfullargspec
 from pathlib import Path
 from typing import Any, Dict, Optional
@@ -46,10 +45,6 @@ class StrategyResolver(IResolver):
             strategy_name, config=config,
             extra_dir=config.get('strategy_path'))
 
-        # make sure ask_strategy dict is available
-        if 'ask_strategy' not in config:
-            config['ask_strategy'] = {}
-
         if hasattr(strategy, 'ticker_interval') and not hasattr(strategy, 'timeframe'):
             # Assign ticker_interval to timeframe to keep compatibility
             if 'timeframe' not in config:
@@ -58,45 +53,54 @@ class StrategyResolver(IResolver):
                     )
                 strategy.timeframe = strategy.ticker_interval
 
+        if strategy._ft_params_from_file:
+            # Set parameters from Hyperopt results file
+            params = strategy._ft_params_from_file
+            strategy.minimal_roi = params.get('roi', strategy.minimal_roi)
+
+            strategy.stoploss = params.get('stoploss', {}).get('stoploss', strategy.stoploss)
+            trailing = params.get('trailing', {})
+            strategy.trailing_stop = trailing.get('trailing_stop', strategy.trailing_stop)
+            strategy.trailing_stop_positive = trailing.get('trailing_stop_positive',
+                                                           strategy.trailing_stop_positive)
+            strategy.trailing_stop_positive_offset = trailing.get(
+                'trailing_stop_positive_offset', strategy.trailing_stop_positive_offset)
+            strategy.trailing_only_offset_is_reached = trailing.get(
+                'trailing_only_offset_is_reached', strategy.trailing_only_offset_is_reached)
+
         # Set attributes
         # Check if we need to override configuration
         #             (Attribute name,                    default,     subkey)
-        attributes = [("minimal_roi",                     {"0": 10.0}, None),
-                      ("timeframe",                       None,        None),
-                      ("stoploss",                        None,        None),
-                      ("trailing_stop",                   None,        None),
-                      ("trailing_stop_positive",          None,        None),
-                      ("trailing_stop_positive_offset",   0.0,         None),
-                      ("trailing_only_offset_is_reached", None,        None),
-                      ("use_custom_stoploss",             None,        None),
-                      ("process_only_new_candles",        None,        None),
-                      ("order_types",                     None,        None),
-                      ("order_time_in_force",             None,        None),
-                      ("stake_currency",                  None,        None),
-                      ("stake_amount",                    None,        None),
-                      ("protections",                     None,        None),
-                      ("startup_candle_count",            None,        None),
-                      ("unfilledtimeout",                 None,        None),
-                      ("use_sell_signal",                 True,        'ask_strategy'),
-                      ("sell_profit_only",                False,       'ask_strategy'),
-                      ("ignore_roi_if_buy_signal",        False,       'ask_strategy'),
-                      ("sell_profit_offset",              0.0,         'ask_strategy'),
-                      ("disable_dataframe_checks",        False,       None),
-                      ("ignore_buying_expired_candle_after",  0,       'ask_strategy')
+        attributes = [("minimal_roi",                     {"0": 10.0}),
+                      ("timeframe",                       None),
+                      ("stoploss",                        None),
+                      ("trailing_stop",                   None),
+                      ("trailing_stop_positive",          None),
+                      ("trailing_stop_positive_offset",   0.0),
+                      ("trailing_only_offset_is_reached", None),
+                      ("use_custom_stoploss",             None),
+                      ("process_only_new_candles",        None),
+                      ("order_types",                     None),
+                      ("order_time_in_force",             None),
+                      ("stake_currency",                  None),
+                      ("stake_amount",                    None),
+                      ("protections",                     None),
+                      ("startup_candle_count",            None),
+                      ("unfilledtimeout",                 None),
+                      ("use_sell_signal",                 True),
+                      ("sell_profit_only",                False),
+                      ("ignore_roi_if_buy_signal",        False),
+                      ("sell_profit_offset",              0.0),
+                      ("disable_dataframe_checks",        False),
+                      ("ignore_buying_expired_candle_after",  0)
                       ]
-        for attribute, default, subkey in attributes:
-            if subkey:
-                StrategyResolver._override_attribute_helper(strategy, config.get(subkey, {}),
-                                                            attribute, default)
-            else:
-                StrategyResolver._override_attribute_helper(strategy, config,
-                                                            attribute, default)
+        for attribute, default in attributes:
+            StrategyResolver._override_attribute_helper(strategy, config,
+                                                        attribute, default)
 
         # Loop this list again to have output combined
-        for attribute, _, subkey in attributes:
-            if subkey and attribute in config[subkey]:
-                logger.info("Strategy using %s: %s", attribute, config[subkey][attribute])
-            elif attribute in config:
+        for attribute, _ in attributes:
+            if attribute in config:
                 logger.info("Strategy using %s: %s", attribute, config[attribute])
 
         StrategyResolver._normalize_attributes(strategy)
@@ -114,7 +118,9 @@ class StrategyResolver(IResolver):
         - Strategy
         - default (if not None)
         """
-        if attribute in config:
+        if (attribute in config
+                and not isinstance(getattr(type(strategy), 'my_property', None), property)):
+            # Ensure Properties are not overwritten
             setattr(strategy, attribute, config[attribute])
             logger.info("Override strategy '%s' with value in config file: %s.",
                         attribute, config[attribute])
@@ -139,7 +145,7 @@ class StrategyResolver(IResolver):
 
         # Sort and apply type conversions
         if hasattr(strategy, 'minimal_roi'):
-            strategy.minimal_roi = OrderedDict(sorted(
+            strategy.minimal_roi = dict(sorted(
                 {int(key): value for (key, value) in strategy.minimal_roi.items()}.items(),
                 key=lambda t: t[0]))
         if hasattr(strategy, 'stoploss'):
