@@ -1,3 +1,4 @@
+import logging
 from copy import deepcopy
 from pathlib import Path
 from typing import List, Optional
@@ -22,6 +23,8 @@ from freqtrade.rpc.api_server.deps import get_config, get_rpc, get_rpc_optional
 from freqtrade.rpc.rpc import RPCException
 
 
+logger = logging.getLogger(__name__)
+
 # Public API, requires no auth.
 router_public = APIRouter()
 # Private API, protected by authentication
@@ -162,8 +165,8 @@ def delete_lock_pair(payload: DeleteLockRequest, rpc: RPC = Depends(get_rpc)):
 
 
 @router.get('/logs', response_model=Logs, tags=['info'])
-def logs(limit: Optional[int] = None, rpc: RPC = Depends(get_rpc)):
-    return rpc._rpc_get_logs(limit)
+def logs(limit: Optional[int] = None):
+    return RPC._rpc_get_logs(limit)
 
 
 @router.post('/start', response_model=StatusMsg, tags=['botcontrol'])
@@ -249,7 +252,7 @@ def list_available_pairs(timeframe: Optional[str] = None, stake_currency: Option
     pair_interval = sorted(pair_interval, key=lambda x: x[0])
 
     pairs = list({x[0] for x in pair_interval})
-
+    pairs.sort()
     result = {
         'length': len(pairs),
         'pairs': pairs,
