@@ -5,8 +5,10 @@ This module defines a base class for auto-hyperoptable strategies.
 import logging
 from abc import ABC, abstractmethod
 from contextlib import suppress
+from pathlib import Path
 from typing import Any, Dict, Iterator, List, Optional, Sequence, Tuple, Union
 
+from freqtrade.misc import deep_merge_dicts, json_load
 from freqtrade.optimize.hyperopt_tools import HyperoptTools
 
 
@@ -14,8 +16,8 @@ with suppress(ImportError):
     from skopt.space import Integer, Real, Categorical
     from freqtrade.optimize.space import SKDecimal
 
+from freqtrade.enums import RunMode
 from freqtrade.exceptions import OperationalException
-from freqtrade.state import RunMode
 
 
 logger = logging.getLogger(__name__)
@@ -205,6 +207,21 @@ class DecimalParameter(NumericParameter):
         return SKDecimal(low=self.low, high=self.high, decimals=self._decimals, name=name,
                          **self._space_params)
 
+    @property
+    def range(self):
+        """
+        Get each value in this space as list.
+        Returns a List from low to high (inclusive) in Hyperopt mode.
+        Returns a List with 1 item (`value`) in "non-hyperopt" mode, to avoid
+        calculating 100ds of indicators.
+        """
+        if self.in_space and self.optimize:
+            low = int(self.low * pow(10, self._decimals))
+            high = int(self.high * pow(10, self._decimals)) + 1
+            return [round(n * pow(0.1, self._decimals), self._decimals) for n in range(low, high)]
+        else:
+            return [self.value]
+
 
 class CategoricalParameter(BaseParameter):
     default: Any
@@ -239,10 +256,23 @@ class CategoricalParameter(BaseParameter):
         """
         return Categorical(self.opt_range, name=name, **self._space_params)
 
+    @property
+    def range(self):
+        """
+        Get each value in this space as list.
+        Returns a List of categories in Hyperopt mode.
+        Returns a List with 1 item (`value`) in "non-hyperopt" mode, to avoid
+        calculating 100ds of indicators.
+        """
+        if self.in_space and self.optimize:
+            return self.opt_range
+        else:
+            return [self.value]
+
 
 class HyperStrategyMixin(object):
     """
-    A helper base class which allows HyperOptAuto class to reuse implementations of of buy/sell
+    A helper base class which allows HyperOptAuto class to reuse implementations of buy/sell
      strategy logic.
     """
 
@@ -258,7 +288,7 @@ class HyperStrategyMixin(object):
 
     def enumerate_parameters(self, category: str = None) -> Iterator[Tuple[str, BaseParameter]]:
         """
-        Find all optimizeable parameters and return (name, attr) iterator.
+        Find all optimizable parameters and return (name, attr) iterator.
         :param category:
         :return:
         """
@@ -273,11 +303,12 @@ class HyperStrategyMixin(object):
         for par in params:
             yield par.name, par
 
-    def _detect_parameters(self, category: str) -> Iterator[Tuple[str, BaseParameter]]:
+    @classmethod
+    def detect_parameters(cls, category: str) -> Iterator[Tuple[str, BaseParameter]]:
         """ Detect all parameters for 'category' """
-        for attr_name in dir(self):
+        for attr_name in dir(cls):
             if not attr_name.startswith('__'):  # Ignore internals, not strictly necessary.
-                attr = getattr(self, attr_name)
+                attr = getattr(cls, attr_name)
                 if issubclass(attr.__class__, BaseParameter):
                     if (attr_name.startswith(category + '_')
                             and attr.category is not None and attr.category != category):
@@ -287,23 +318,62 @@ class HyperStrategyMixin(object):
                             (attr_name.startswith(category + '_') and attr.category is None)):
                         yield attr_name, attr
 
+    @classmethod
+    def detect_all_parameters(cls) -> Dict:
+        """ Detect all parameters and return them as a list"""
+        params: Dict = {
+            'buy': list(cls.detect_parameters('buy')),
+            'sell': list(cls.detect_parameters('sell')),
+        }
+        params.update({
+            'count': len(params['buy'] + params['sell'])
+        })
+
+        return params
+
     def _load_hyper_params(self, hyperopt: bool = False) -> None:
         """
         Load Hyperoptable parameters
         """
-        self._load_params(getattr(self, 'buy_params', None), 'buy', hyperopt)
-        self._load_params(getattr(self, 'sell_params', None), 'sell', hyperopt)
+        params = self.load_params_from_file()
+        params = params.get('params', {})
+        self._ft_params_from_file = params
+        buy_params = deep_merge_dicts(params.get('buy', {}), getattr(self, 'buy_params', {}))
+        sell_params = deep_merge_dicts(params.get('sell', {}), getattr(self, 'sell_params', {}))
 
-    def _load_params(self, params: dict, space: str, hyperopt: bool = False) -> None:
+        self._load_params(buy_params, 'buy', hyperopt)
+        self._load_params(sell_params, 'sell', hyperopt)
+
+    def load_params_from_file(self) -> Dict:
+        filename_str = getattr(self, '__file__', '')
+        if not filename_str:
+            return {}
+        filename = Path(filename_str).with_suffix('.json')
+
+        if filename.is_file():
+            logger.info(f"Loading parameters from file {filename}")
+            try:
+                params = json_load(filename.open('r'))
+                if params.get('strategy_name') != self.__class__.__name__:
+                    raise OperationalException('Invalid parameter file provided.')
+                return params
+            except ValueError:
+                logger.warning("Invalid parameter file format.")
+                return {}
+        logger.info("Found no parameter file.")
+
+        return {}
+
+    def _load_params(self, params: Dict, space: str, hyperopt: bool = False) -> None:
         """
-        Set optimizeable parameter values.
+        Set optimizable parameter values.
         :param params: Dictionary with new parameter values.
         """
         if not params:
             logger.info(f"No params for {space} found, using default values.")
         param_container: List[BaseParameter] = getattr(self, f"ft_{space}_params")
 
-        for attr_name, attr in self._detect_parameters(space):
+        for attr_name, attr in self.detect_parameters(space):
             attr.name = attr_name
             attr.in_space = hyperopt and HyperoptTools.has_space(self.config, space)
             if not attr.category:
@@ -321,7 +391,7 @@ class HyperStrategyMixin(object):
             else:
                 logger.info(f'Strategy Parameter(default): {attr_name} = {attr.value}')
 
-    def get_params_dict(self):
+    def get_no_optimize_params(self):
         """
         Returns list of Parameters that are not part of the current optimize job
         """
